/*
 * ar4c-dcdc - adc-avr.c
 *
 * atmega ADC driver
 *
 * Copyright (C) 2014  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <compiler.h>
#include <adc.h>
#include <avr/io.h>

#define ADMUX_VREF_AREF	0x00
#define ADMUX_VREF_INT	0xc0
#define ADMUX_ADLAR	0x20
#define ADMUX_MUX_MASK	0x0f

#define ADMUX_CONFIG		(ADMUX_VREF_AREF)

#define ADCSRA_DIV_2	0x01
#define ADCSRA_DIV_4	0x02
#define ADCSRA_DIV_8	0x03
#define ADCSRA_DIV_16	0x04
#define ADCSRA_DIV_32	0x05
#define ADCSRA_DIV_64	0x06
#define ADCSRA_DIV_128	0x07

void adc_init(void)
{
	/*
	 * The ADC frequency should be in 50 kHz to 200 kHz range.
	 */
	ADCSRA = BIT(ADEN) | ADCSRA_DIV_8;

	ADMUX = ADMUX_CONFIG;
}

unsigned int adc_get(unsigned char ch)
{
	unsigned char x;

	x = ADMUX;
	x &= ~ADMUX_MUX_MASK;
	x |= ch & ADMUX_MUX_MASK;
	ADMUX = x;

	ADCSRA |= BIT(ADSC);
	do {
		cpu_relax();
	} while (ADCSRA & BIT(ADSC));
	return ADC;
}
