/*
 * ar4c-dcdc - jiffies.c
 *
 * Copyright (C) 2014  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <jiffies.h>
#include <compiler.h>
#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>

volatile unsigned char jiffies = -HZ;

ISR(TIM0_COMPA_vect)
{
	jiffies++;
}

void sleep_to(unsigned char t)
{
	set_sleep_mode(SLEEP_MODE_IDLE);
	cli();
	while (time_after_eq(t, jiffies)) {
		sleep_enable();
		sei();
		sleep_cpu();
		sleep_disable();
		cli();
	}
	sei();
}

#define TCCR_DIV1		1
#define TCCR_DIV8		2
#define TCCR_DIV64		3
#define TCCR_DIV256		4
#define TCCR_DIV1024		5

void __init jiffies_init(void)
{
	TCCR0A = BIT(WGM01);
	TCCR0B = TCCR_DIV256;
	OCR0A = 75;
	TCNT0 = 0;
	TIMSK0 = BIT(OCIE0A);
	sei();
}
